########################################################################
# AoV.tcl - Manages f/Calc's "Angle of View" tab.
########################################################################


########################################################################
## Event Handlers
########################################################################

proc aov_FilmFormatListChange { name1 name2 op } {
	RecalculateAoV
}

proc aov_FocalLengthEntryChange { name index op } {
	RecalculateAoV
}


########################################################################
## Procedures
########################################################################

#### RecalculateAoV ####################################################
# Recalculates the angle of view

proc RecalculateAoV { } {
	global g_filmFormat
	global g_focalLength
	
	global aov_vertical
	global aov_horizontal
	global aov_diagonal

	if { $g_focalLength <= 0.0 } {
		# Focal length is hosed: we refuse to work under these `orrid
		# conditions.
		set aov_vertical "n/a"
		set aov_horizontal "n/a"
		set aov_diagonal "n/a"
		return
	}

	# Get the film size for this format
	set filmDimensions [ GetFilmDimensions $g_filmFormat ]
	set filmHeight [ lindex $filmDimensions 1 ]
	set filmWidth [ lindex $filmDimensions 0 ]
	set filmDiagonal [ expr sqrt(($filmWidth * $filmWidth) + \
			($filmHeight * $filmHeight)) ]

	# Do the calculation
	set verticalAngle [ AngleOfView $g_focalLength $filmHeight ]
	set horizontalAngle [ AngleOfView $g_focalLength $filmWidth ]
	set diagonalAngle [ AngleOfView $g_focalLength $filmDiagonal ]
	
	# Put the results in the output fields
	set aov_vertical "[ ToDegrees $verticalAngle ]"
	set aov_horizontal "[ ToDegrees $horizontalAngle ]"
	set aov_diagonal "[ ToDegrees $diagonalAngle ]"
}


########################################################################
## Tab setup
########################################################################

#### Add our tab to the top-level notebook
.noteBook add aov -label "Angle of View" -underline 0
set aovTab [.noteBook subwidget aov]


#### Create the tab's controls
label $aovTab.promptLabel \
		-text [ concat "Calculate the angle of view for a lens and " \
		"film format, at infinity:" ]
frame $aovTab.spacer1
tixComboBox $aovTab.filmFormatList -label "Film format: " \
		-dropdown true -editable false -variable g_filmFormat \
		-options { \
			listbox.height 6
			label.anchor e
			label.width 18
			entry.width 10
		}
tixLabelEntry $aovTab.focalLengthEntry -label "Focal length (mm): " \
		-options { \
			entry.width 10 
			label.width 18
			label.anchor e 
			entry.textVariable g_focalLength
		}
frame $aovTab.spacer2
label $aovTab.resultLabel -text "Resulting angle of view:"
frame $aovTab.frame1
tixLabelEntry $aovTab.frame1.verticalField \
		-label "Vertical: " -options { \
			entry.width 16 
			label.width 18
			label.anchor e 
			entry.textVariable aov_vertical
		}
tixLabelEntry $aovTab.frame1.horizontalField \
		-label "Horizontal: " -options { \
			entry.width 16 
			label.anchor e 
			entry.textVariable aov_horizontal
		}
tixLabelEntry $aovTab.diagonalField \
		-label "Diagonal: " -options { \
			entry.width 16 
			label.width 18
			label.anchor e 
			entry.textVariable aov_diagonal
		}


#### Fill controls with defaults
FillFormatList $aovTab.filmFormatList
tixSetSilent $aovTab.filmFormatList "35mm"
RecalculateAoV


#### Pack controls into tab
pack $aovTab.promptLabel -side top -anchor nw -pady 2
pack $aovTab.spacer1 -side top -anchor nw -pady 5
pack $aovTab.filmFormatList -side top -anchor nw -pady 2
pack $aovTab.focalLengthEntry -side top -anchor nw -pady 2
pack $aovTab.spacer2 -side top -anchor nw -pady 5
pack $aovTab.resultLabel -side top -anchor nw -pady 2
pack $aovTab.frame1.verticalField -side left -anchor nw
pack $aovTab.frame1.horizontalField -side left -anchor nw -padx 8
pack $aovTab.frame1 -side top -anchor nw -pady 2
pack $aovTab.diagonalField -side top -anchor nw -pady 2


#### Disable output-only controls
set verticalField [ $aovTab.frame1.verticalField subwidget entry ]
set horizontalField [ $aovTab.frame1.horizontalField subwidget entry ]
set diagonalField [ $aovTab.diagonalField subwidget entry ]
$verticalField configure -state disabled
$horizontalField configure -state disabled
$diagonalField configure -state disabled


#### Set up "control changed" callbacks
trace variable g_focalLength w aov_FocalLengthEntryChange
trace variable g_filmFormat w aov_FilmFormatListChange

