########################################################################
# FoV.tcl - Manages f/Calc's "Field of View" tab.
########################################################################


########################################################################
## Event Handlers
########################################################################

proc fov_UnitRadioClicked { } {
	global g_previousUnit
	global g_distanceUnit
	global g_subjectDistance

	if { $g_previousUnit != $g_distanceUnit } {
		set mm [ ToMillimeters $g_subjectDistance $g_previousUnit ]
		set g_subjectDistance \
				[ FromMillimeters $mm $g_distanceUnit ]
		set g_previousUnit $g_distanceUnit
	}
}

proc fov_SubjectDistanceEntryChange { name index op } {
	RecalculateFoV
}

proc fov_FocalLengthEntryChange { name index op } {
	RecalculateFoV
}

proc fov_FilmFormatListChange { name1 name2 op } {
	RecalculateFoV
}


########################################################################
## Procedures
########################################################################

#### RecalculateFoV ####################################################
# Recalculates the field of view

proc RecalculateFoV { } {
	global g_subjectDistance
	global g_focalLength
	global g_filmFormat
	
	global g_distanceUnit

	global fov_vertical
	global fov_horizontal

	set subjectDistance [ ToMillimeters $g_subjectDistance \
			$g_distanceUnit ] 

	set sane true
	if { $g_focalLength <= 0.0 } {
		set sane false
	} elseif { $subjectDistance < $g_focalLength } {
		set sane false
	}
	if { $sane == "false" } {
		set fov_vertical "n/a"
		set fov_horizontal "n/a"
		return
	}

	# Calculate the subject magnification
	set magnification [ Magnification $subjectDistance \
			$g_focalLength ]

	# Get the film size for this format
	set filmDimensions [ GetFilmDimensions $g_filmFormat ]
	set filmHeight [ lindex $filmDimensions 1 ]
	set filmWidth [ lindex $filmDimensions 0 ]

	# Calculate subject size based on magnification and film size.
	set verticalSize [ FromMillimeters \
			[ expr $filmHeight / $magnification ] \
			$g_distanceUnit ]
	set horizontalSize [ FromMillimeters \
			[ expr $filmWidth / $magnification ] \
			$g_distanceUnit ]

	# Put the calculated values into the output fields.
	set fov_vertical "$verticalSize $g_distanceUnit"
	set fov_horizontal "$horizontalSize $g_distanceUnit"
}


########################################################################
## Tab setup
########################################################################

#### Add our tab to the top-level notebook
.noteBook add fov -label "Field of View" -underline 0
set fovTab [.noteBook subwidget fov]


#### Create the tab's controls
label $fovTab.promptLabel \
		-text [ concat "Calculate the maximum subject size, given the " \
		"subject distance, the focal length of the lens and the film\n" \
		"format:" ] -justify left
frame $fovTab.spacer1
frame $fovTab.frame1
tixLabelEntry $fovTab.frame1.subjectDistanceEntry \
		-label "Subject distance: " -options { \
			entry.width 8 
			label.width 16
			label.anchor e 
			entry.textVariable g_subjectDistance
		}
radiobutton $fovTab.frame1.metersRadio -command fov_UnitRadioClicked \
		-text "m" -variable g_distanceUnit -value m 
radiobutton $fovTab.frame1.centimetersRadio -command fov_UnitRadioClicked \
		-text "cm" -variable g_distanceUnit -value cm
radiobutton $fovTab.frame1.feetRadio -command fov_UnitRadioClicked \
		-text "ft" -variable g_distanceUnit -value ft 
radiobutton $fovTab.frame1.inchesRadio -command fov_UnitRadioClicked \
		-text "in" -variable g_distanceUnit -value in
tixLabelEntry $fovTab.focalLengthEntry -label "Focal length (mm): " \
		-options { \
			entry.width 8 
			label.width 16
			label.anchor e 
			entry.textVariable g_focalLength
		}
tixComboBox $fovTab.filmFormatList -label "Film format: " \
		-dropdown true -editable false -variable g_filmFormat \
		-options { \
			listbox.height 6
			label.anchor e
			label.width 16
		}
frame $fovTab.frame2
label $fovTab.frame2.resultLabel -text "Resulting FoV: "
tixLabelEntry $fovTab.frame2.verticalField -label "Vertical: " \
		-options { \
			entry.width 18
			label.anchor w
			entry.textVariable fov_vertical
		}
tixLabelEntry $fovTab.frame2.horizontalField -label "Horizontal: " \
		-options { \
			entry.width 18
			label.anchor w
			entry.textVariable fov_horizontal
		}


#### Fill controls with defaults
$fovTab.frame1.metersRadio select
FillFormatList $fovTab.filmFormatList
tixSetSilent $fovTab.filmFormatList "35mm"
RecalculateFoV


#### Pack controls into tab
pack $fovTab.promptLabel -side top -anchor nw -pady 2
pack $fovTab.spacer1 -side top -anchor nw -pady 5
pack $fovTab.frame1.subjectDistanceEntry -side left -anchor nw
pack $fovTab.frame1.metersRadio -side left -anchor nw
pack $fovTab.frame1.centimetersRadio -side left -anchor nw
pack $fovTab.frame1.feetRadio -side left -anchor nw
pack $fovTab.frame1.inchesRadio -side left -anchor nw
pack $fovTab.frame1 -side top -anchor nw -pady 2
pack $fovTab.focalLengthEntry -side top -anchor nw -pady 2
pack $fovTab.filmFormatList -side top -anchor nw -pady 2
pack $fovTab.frame2.resultLabel -side left -anchor w
pack $fovTab.frame2.verticalField -side left -anchor nw -padx 4
pack $fovTab.frame2.horizontalField -side left -anchor nw -padx 4
pack $fovTab.frame2 -side top -anchor nw -pady 15


#### Disable output-only controls
set verticalField [ $fovTab.frame2.verticalField subwidget entry ]
set horizontalField [ $fovTab.frame2.horizontalField subwidget entry ]
$verticalField configure -state disabled
$horizontalField configure -state disabled


#### Set up "control changed" callbacks
trace variable g_subjectDistance w fov_SubjectDistanceEntryChange
trace variable g_focalLength w fov_FocalLengthEntryChange
trace variable g_filmFormat w fov_FilmFormatListChange

