########################################################################
# Hyperfocal.tcl - Manages f/Calc's "Hyperfocal" tab.
########################################################################


########################################################################
## Event Handlers
########################################################################

proc hyp_UnitRadioClicked { } {
	RecalculateHyperfocal
}

proc hyp_FocalLengthEntryChange { name index op } {
	RecalculateHyperfocal
}

proc hyp_ApertureListChange { name1 name2 op } {
	RecalculateHyperfocal
}

proc hyp_CoCEntryChange { name index op } {
	RecalculateHyperfocal
}


########################################################################
## Procedures
########################################################################

#### RecalculateHyperfocal #############################################
# Recalculates the depth-of-field

proc RecalculateHyperfocal { } {
	global g_focalLength
	global g_aperture
	global g_CoC
	
	global g_distanceUnit

	global hyperfocal_result

	set sane true
	if { $g_focalLength <= 0.0 } {
		set sane false
	} elseif { $g_aperture < 0.0 } {
		set sane false
	} elseif { $g_CoC < 0.0 } {
		set sane false
	}
	if { $sane == "false" } {
		set hyperfocal_result "n/a"
		return
	}

	# Do the calculations
	set result [ HyperfocalDistance $g_focalLength \
			$g_aperture $g_CoC ]

	# Convert the result to the user's unit of choice and output it.
	set hyperfocal_result [ FromMillimeters $result \
			$g_distanceUnit ]
	append hyperfocal_result " $g_distanceUnit"
}


########################################################################
## Tab setup
########################################################################

#### Add our tab to the top-level notebook
.noteBook add hyperfocal -label "Hyperfocal" -underline 1
set hyperfocalTab [.noteBook subwidget hyperfocal]


#### Create the tab's controls
label $hyperfocalTab.promptLabel \
		-text [ concat "Calculate the focusing distance that offers the " \
		"deepest in-focus field, for a given focal length, aperture" \
		"and\nfilm format:" ] -justify left
frame $hyperfocalTab.spacer1
tixLabelEntry $hyperfocalTab.focalLengthEntry -label "Focal length (mm): " \
		-options { \
			entry.width 8 
			label.width 22
			label.anchor e 
			entry.textVariable g_focalLength
		}
tixComboBox $hyperfocalTab.apertureList -label "Aperture: " -dropdown true \
		-editable true -variable g_aperture -options { \
			listbox.height 6
			label.width 22
			label.anchor e
		}
frame $hyperfocalTab.frame1
tixLabelEntry $hyperfocalTab.frame1.cocEntry -label "Circle of Confusion (mm): " \
		-options { \
			entry.width 8
			label.width 22
			label.anchor e
			entry.textVariable g_CoC
		}
tixComboBox $hyperfocalTab.frame1.suggestCoCList -label "Suggest CoC for: " \
		-dropdown true -editable false -variable g_filmFormat \
		-options { \
			listbox.height 6
			label.anchor w
		}
frame $hyperfocalTab.frame2
label $hyperfocalTab.frame2.distanceLabel -text "Express distance in: " \
		-width 22 -anchor e
radiobutton $hyperfocalTab.frame2.metersRadio \
		-command hyp_UnitRadioClicked -text "m" \
		-variable g_distanceUnit -value m 
radiobutton $hyperfocalTab.frame2.centimetersRadio \
		-command hyp_UnitRadioClicked -text "cm" \
		-variable g_distanceUnit -value cm
radiobutton $hyperfocalTab.frame2.feetRadio \
		-command hyp_UnitRadioClicked -text "ft" \
		-variable g_distanceUnit -value ft 
radiobutton $hyperfocalTab.frame2.inchesRadio \
		-command hyp_UnitRadioClicked -text "in" \
		-variable g_distanceUnit -value in
tixLabelEntry $hyperfocalTab.resultField -label "Hyperfocal distance: " \
		-options { \
			entry.width 18
			label.width 22
			label.anchor e
			entry.textVariable hyperfocal_result
		}


#### Fill controls with defaults
FillFList $hyperfocalTab.apertureList
tixSetSilent $hyperfocalTab.apertureList "1.8"
FillFormatList $hyperfocalTab.frame1.suggestCoCList
tixSetSilent $hyperfocalTab.frame1.suggestCoCList "35mm"
$hyperfocalTab.frame2.metersRadio select
RecalculateHyperfocal


#### Pack controls into tab
pack $hyperfocalTab.promptLabel -side top -anchor nw -pady 2
pack $hyperfocalTab.spacer1 -side top -anchor nw -pady 5
pack $hyperfocalTab.focalLengthEntry -side top -anchor nw -pady 2
pack $hyperfocalTab.apertureList -side top -anchor nw -pady 2
pack $hyperfocalTab.frame1.cocEntry -side left -anchor nw 
pack $hyperfocalTab.frame1.suggestCoCList -side left -anchor nw -padx 8
pack $hyperfocalTab.frame1 -side top -anchor nw -pady 2
pack $hyperfocalTab.frame2.distanceLabel -side left -anchor nw -pady 2
pack $hyperfocalTab.frame2.metersRadio -side left -anchor nw -padx 4
pack $hyperfocalTab.frame2.centimetersRadio -side left -anchor nw \
		-padx 4
pack $hyperfocalTab.frame2.feetRadio -side left -anchor nw -padx 4
pack $hyperfocalTab.frame2.inchesRadio -side left -anchor nw -padx 4
pack $hyperfocalTab.frame2 -side top -anchor nw -pady 2
pack $hyperfocalTab.resultField -side top -anchor nw -pady 8


#### Disable output-only control
set resultField [ $hyperfocalTab.resultField subwidget entry ]
$resultField configure -state disabled


#### Set up "control changed" callbacks
trace variable g_focalLength w hyp_FocalLengthEntryChange
trace variable g_aperture w hyp_ApertureListChange
trace variable g_CoC w hyp_CoCEntryChange
trace variable g_filmFormat w SuggestCoCListChange

