########################################################################
# LensMath.tcl - Main calculations used by f/Calc
########################################################################


########################################################################
# Constants
########################################################################

# Square root of 2 -- used in aperture formulae
set kSqrt2 [ expr sqrt(2) ] 

# A value we use in f number arithmetic, which we can precalculate.
set kNatLogSqrt2 [ expr log($kSqrt2) ]

# Pi
set kPi 3.141592654


########################################################################
# Globals
########################################################################

## Common film formats
set gFormatList { "APS" "35mm" "6x4.5cm" "6x6cm" "6x7cm"
		"6x9cm" "4x5in" "8x10in" }

## Standard CoC values for the common film formats
set gCoCArray(APS) 0.025
set gCoCArray(35mm) 0.033
set gCoCArray(6x4.5cm) 0.05
set gCoCArray(6x6cm) 0.06
set gCoCArray(6x7cm) 0.065
set gCoCArray(6x9cm) 0.07
set gCoCArray(4x5in) 0.15
set gCoCArray(8x10in) 0.3

## Size in mm of the various film formats
set gFilmDimensions(APS) [ list 30.2 16.7 ]
set gFilmDimensions(35mm) [ list 36.0 24.0 ]
set gFilmDimensions(6x4.5cm) [ list 60.0 45.0 ]
set gFilmDimensions(6x6cm) [ list 60.0 60.0 ]
set gFilmDimensions(6x7cm) [ list 60.0 70.0 ]
set gFilmDimensions(6x9cm) [ list 60.0 90.0 ]
set gFilmDimensions(4x5in) [ list 101.6 127.0 ]
set gFilmDimensions(8x10in) [ list 203.2 254.0 ]

			
#### FillFormatList ####################################################
# Given a tixComboBox, fill its drop-down list with the film formats
# we recognize.

proc FillFormatList { w } {
	global gFormatList

	foreach element $gFormatList { 
		$w insert end $element
	}
}


#### GetCoCForFormat ###################################################
# Given one of the formats from the list generated by FillFormatList,
# return the default circle of confusion for that format.

proc GetCoCForFormat { format } {
	global gFormatList
	global gCoCArray

	foreach element $gFormatList { 
		if { $element == $format } {
			return $gCoCArray($format)
		}
	}

	return "0.0"
}


#### FillFList #########################################################
# Given a tixComboBox, fill its drop-down list with a series of "whole"
# f-numbers (e.g. 1.0, 1.414, 2.0, 2.828, etc.)  Inserts numbers from 
# 1.0 to 64.0.

proc FillFList { w } {
	set start 0
	set end 13
	global kSqrt2

	for { set i $start } { $i < $end } { incr i } {
		$w insert end [ expr pow($kSqrt2, $i) ]
	}
}


#### AdjustAperture ####################################################
# Given an initial aperture value and a count of stops, return the new
# aperture value.  Returns 0 if fInitialAperture is also 0.

proc AdjustAperture { initialAperture numStops } {
	global kNatLogSqrt2
	global kSqrt2

	if [ expr $initialAperture > 0.0 ] {
		# Calculate number of stops from f/1.0 that new stop will be,
		# turn that back into an f value, and return it.
		set newStopCount [ expr \
				(log($initialAperture) / $kNatLogSqrt2) + $numStops ]
		return [ expr pow($kSqrt2, $newStopCount) ]
	} else {
		return 0.0
	}
}


#### ApertureDelta #####################################################
# Calculates the distance between two aperture values, in stops.
# (f2 - f1).

proc ApertureDelta { f1 f2 } {
	global kNatLogSqrt2
	
	set f1StopDistance [ expr log($f1) / $kNatLogSqrt2 ]
	set f2StopDistance [ expr log($f2) / $kNatLogSqrt2 ]
	return [ expr $f2StopDistance - $f1StopDistance ]
}


#### HyperfocalDistance ################################################
# Calculates the hyperfocal distance of a lens, given the focal length
# of the lens, the aperture and the circle of confusion.  All measure-
# ments must be in the same unit, such as millimeters.

proc HyperfocalDistance { focalLength aperture coc } {
	return [ expr ($focalLength * $focalLength) / ($aperture * $coc) ]
}


#### DepthOfField ######################################################
# Returns a list containing the near and far edges of the in-focus
# field, given the focal length of the lens, the aperture, the circle
# of confusion and the subject distance.  All measurements must be in 
# the same unit, such as millimeters.

proc DepthOfField { focalLength aperture coc subjectDistance } {
	set hyperfocal [ HyperfocalDistance $focalLength $aperture $coc ]
	set numerator [ expr $hyperfocal * $subjectDistance ]
	set lengthDelta [ expr $subjectDistance - $focalLength ]
	set dofNear [ expr $numerator / ($hyperfocal + $lengthDelta) ]
	set dofFar [ expr $numerator / ($hyperfocal - $lengthDelta) ]

	return [ list $dofNear $dofFar ]
}


#### FromMillimeters ###################################################
# Converts a value from millimeters to any of several unit types.

proc FromMillimeters { distance unit } {
	if { $unit == "ft" } {
		return [ expr $distance / 304.8 ]
	} elseif { $unit == "in" } {
		return [ expr $distance / 25.4 ]
	} elseif { $unit == "cm" } {
		return [ expr $distance / 10.0 ]
	} elseif { $unit == "m" } {
		return [ expr $distance / 1000.0 ]
	} else {
		# User passed something weird in as the unit for some reason, so
		# just echo their input back to them.
		return $distance
	}
}


#### ToMillimeters #####################################################
# Converts a value from any of several measurements to millimeters.

proc ToMillimeters { distance unit } {
	if { $unit == "ft" } {
		return [ expr $distance * 304.8 ]
	} elseif { $unit == "in" } {
		return [ expr $distance * 25.4 ]
	} elseif { $unit == "cm" } {
		return [ expr $distance * 10.0 ]
	} elseif { $unit == "m" } {
		return [ expr $distance * 1000.0 ]
	} else {
		# User passed something weird in as the unit for some reason, so
		# just echo their input back to them.
		return $distance
	}
}


#### Magnification #####################################################
# Calculates the on-film magnification factor given So and the focal 
# length.

proc Magnification { so focalLength } {
	return [ expr $focalLength / ($so - $focalLength) ]
}


#### GetFilmDimensions #################################################
# Returns the dimensions, in millimeters, of several popular film
# formats.  Parameter is one of the values from the gFormatList list.
# Return value is a list: { width height }

proc GetFilmDimensions { filmFormat } {
	global gFilmDimensions

	return $gFilmDimensions($filmFormat)
}


#### AngleOfView #######################################################
# Calculates the minimum angle of view of a lens given the film's size.
# E.g., if filmSize is the width of the film, the returned value is the 
# horizontal angle of view.  Both values must be in the same unit, such
# as millimeters.  This calculation assumes that magnification is
# essentially 0, meaning that the object is fairly distant, which in
# turn yields the minimum angle of view.

proc AngleOfView { focalLength filmSize } {
	return [ expr 2.0 * atan($filmSize / (2.0 * $focalLength)) ]
}


#### ToDegrees #########################################################
# Converts a value in radians to degrees

proc ToDegrees { radians } {
	return [ expr $radians * 57.295780 ]
}


#### MeteringAngle #####################################################
# Calculate the angle that a spot meter of a given frame percentage
# sees, given a focal length and the area of the frame.  This is the
# same basic formula as AngleOfView, but with different intent, and
# different initial inputs.

proc MeteringAngle { frameArea framePercentage focalLength } {
	global kPi
	
	set meteringArea [ expr $frameArea * ($framePercentage / 100.0) ]
	set meterDiameter [ expr 2.0 * sqrt($meteringArea / $kPi) ]
	return [ expr atan($meterDiameter / $focalLength) ]
}


#### RoundToFraction ###################################################
# Rounds f to the nearest 1/frac.  If you don't pass frac, it rounds
# to the nearest integer, or rather to the nearest 1/1.  You can also
# round to the nearest, say, multiple of 10 by passing 0.1.  Beware of
# roundoff errors, though!

proc RoundToFraction { initial { frac 1.0 } } {
	set temp [ expr int(($initial * $frac) + \
			($initial > 0 ? 0.5 : -0.5)) ]
	return [ expr double($temp) / $frac ]
}


